package implement

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/dao"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models"
	db "code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/src/models/official/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_model.git/super_cloud_issuance/enum"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_model.git/utils/logx"
	"strconv"
	"time"
	"xorm.io/xorm"
)

func NewSuperCloudIssuanceUserRobotDb(engine *xorm.Engine, uid int, masterId string) dao.SuperCloudIssuanceUserRobotDao {
	return &SuperCloudIssuanceUserRobotDb{
		Db:       engine,
		Uid:      uid,
		MasterId: masterId,
	}
}

type SuperCloudIssuanceUserRobotDb struct {
	Db       *xorm.Engine `json:"model"`
	Uid      int          `json:"uid"`
	MasterId string       `json:"master_id"`
}

func (superCloudIssuanceUserRobotDb *SuperCloudIssuanceUserRobotDb) InsertCloudIssuanceUserRobot(m *models.SuperCloudIssuanceUserRobot, officialDb *xorm.Engine) (id int, err error) {
	_, err = superCloudIssuanceUserRobotDb.Db.InsertOne(m)
	if err != nil {
		return 0, err
	}
	id = m.Id
	//顺便新增一份数据到总后台的 `super_cloud_issuance_robot_records` 表
	db.CloudIssuanceRobotRecordsInsert(officialDb, &model.SuperCloudIssuanceRobotRecords{
		MasterId: superCloudIssuanceUserRobotDb.MasterId,
		RobotId:  strconv.Itoa(m.RobotId),
		Uid:      superCloudIssuanceUserRobotDb.Uid,
	})
	return id, nil
}

func (superCloudIssuanceUserRobotDb *SuperCloudIssuanceUserRobotDb) GetCloudIssuanceUserFreeRobot() (m *models.SuperCloudIssuanceUserRobot, err error) {
	//TODO::先将所有已过期的状态改变
	timeStamp := time.Now().Unix()
	sql := "update cloud_issuance_user_robot set is_enabled = ? where end_time <= ? and robot_kind =?"
	_, err = superCloudIssuanceUserRobotDb.Db.Exec(sql, 3, timeStamp, enum.ShareRobot)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}

	m = new(models.SuperCloudIssuanceUserRobot)
	has, err := superCloudIssuanceUserRobotDb.Db.Where("uid = -1").
		And("robot_kind =?", enum.ShareRobot).
		And("is_enabled = 0").Get(m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (superCloudIssuanceUserRobotDb *SuperCloudIssuanceUserRobotDb) GetSuperCloudIssuanceUserRobot() (m *models.SuperCloudIssuanceUserRobot, err error) {
	m = new(models.SuperCloudIssuanceUserRobot)
	has, err := superCloudIssuanceUserRobotDb.Db.Where("uid = ?", superCloudIssuanceUserRobotDb.Uid).
		And("is_enabled = 0").Get(m)
	if err != nil {
		return nil, zhios_order_relate_logx.Error(err)
	}
	if has == false {
		return nil, nil
	}
	return m, nil
}

func (superCloudIssuanceUserRobotDb *SuperCloudIssuanceUserRobotDb) SaveSuperCloudIssuanceUserRobot(id interface{}, m *models.SuperCloudIssuanceUserRobot, forceColums ...string) (affected int64, err error) {
	if forceColums != nil {
		affected, err = superCloudIssuanceUserRobotDb.Db.Where("id=?", id).Cols(forceColums...).Update(m)
	} else {
		affected, err = superCloudIssuanceUserRobotDb.Db.Where("id=?", id).Update(m)
	}
	return
}
